/*
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package fr.loof.fonzie.dialects;


import org.hibernate.proxy.HibernateProxy;
import org.hibernate.proxy.LazyInitializer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * @author <a href="nicolas@apache.org">Nicolas De Loof</a>
 */
public class HibernateDialect
    extends JPADialectSupport
{
    protected static Logger logger = LoggerFactory.getLogger( HibernateDialect.class );

    public Object getIdentifier( Object entity )
    {
        if ( isProxy( entity ) )
        {
            logger.debug( "entity {} is an Hibernate Proxy", entity );
            LazyInitializer initializer = ( (HibernateProxy) entity ).getHibernateLazyInitializer();
            return initializer.getIdentifier();
        }
        return readIdentifier( entity );
    }

    public Class getActualType( Object entity )
    {
        if ( isProxy( entity ) )
        {
            logger.debug( "entity {} is an Hibernate Proxy", entity );
            LazyInitializer initializer = ( (HibernateProxy) entity ).getHibernateLazyInitializer();
            return initializer.getPersistentClass();
        }
        return entity.getClass();
    }

    @SuppressWarnings( "unchecked" )
    public <T> T getImplementation( T entity )
    {
        if ( isProxy( entity ) )
        {
            logger.debug( "entity {} is an Hibernate Proxy", entity );
            LazyInitializer initializer = ( (HibernateProxy) entity ).getHibernateLazyInitializer();
            entity = (T) initializer.getImplementation();
        }
        return entity;
    }

    public boolean isProxy( Object entity )
    {
        return entity instanceof HibernateProxy;
    }
}
