/*
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package fr.loof.fonzie.keywords;

import java.util.Collection;
import java.util.ListIterator;

import fr.loof.fonzie.parsers.Constraint;
import fr.loof.fonzie.parsers.DelegateProperty;
import fr.loof.fonzie.parsers.Property;
import fr.loof.fonzie.parsers.QueryElement;

public class In
    extends AbstractConstraint
    implements Constraint
{
    public In()
    {
        super( "in" );
    }

    @Override
    public boolean postProcess( ListIterator<QueryElement> elements )
    {
        elements.previous(); // current
        int count = 1;
        while ( elements.hasPrevious() )
        {
            QueryElement e = elements.previous();
            count++;
            if ( e instanceof Property )
            {
                elements.set( new InProperty( (Property) e ) );
                break;
            }
        }

        while ( count-- > 0 )
        {
            elements.next();
        }
        return true;
    }

    private class InProperty
        extends DelegateProperty
    {
        public InProperty( Property property )
        {
            super( property );
        }

        @Override
        public boolean doCheckType( Class parameterType )
        {
            if ( parameterType.isArray() )
            {
                Class type = parameterType.getComponentType();
                return super.doCheckType( type );
            }
            if ( parameterType.isAssignableFrom( Collection.class ) )
            {
                // No way to check collection type :'(
                return true;
            }
            return false;
        }
    }

}