/*
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package fr.loof.fonzie.keywords;


import java.util.List;
import java.util.ListIterator;

import fr.loof.fonzie.parsers.DelegateProperty;
import fr.loof.fonzie.parsers.JPQLPersistenceQuery;
import fr.loof.fonzie.parsers.KeyWord;
import fr.loof.fonzie.parsers.Property;
import fr.loof.fonzie.parsers.QueryElement;

public class Join
    extends KeyWord
{
    @Override
    public void process( JPQLPersistenceQuery query, List<Class> params )
    {
        query.appendSelect( " join" );
    }

    @Override
    public boolean postProcess( ListIterator<QueryElement> elements )
    {
        if ( !elements.hasNext() )
        {
            throw new IllegalStateException( "Join keyword must be followed by an entity property name" );
        }
        QueryElement e = elements.next();
        if ( e instanceof Property )
        {
            elements.set( new JoinProperty( (Property) e ) );
        }

        return true;
    }

    private class JoinProperty
        extends DelegateProperty
    {
        private JoinProperty( Property property )
        {
            super( property );
        }

        public boolean postProcess( ListIterator<QueryElement> elements )
        {
            return false;
        }

        public void process( JPQLPersistenceQuery query, List<Class> params )
        {
            query.appendSelect( " " )
                 .appendSelect( query.getCurrentEntityShortName() )
                 .appendSelect( "." )
                 .appendSelect( getName() );
            Class propertyType = KeyWord.checkFieldExists( query.getCurrentEntity(), getName() );
            query.setCurrentEntity( propertyType );
            query.appendSelect( " " ).appendSelect( query.getCurrentEntityShortName() );
        }
    }
}