/*
 *  Licensed to the Apache Softwimport org.apache.commons.lang.StringUtils;

import fr.loof.fonzie.JPQLPersistenceQuery;
file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package fr.loof.fonzie.parsers;

import java.lang.reflect.Field;
import java.util.List;
import java.util.ListIterator;

import org.springframework.util.ReflectionUtils;


public abstract class KeyWord
    implements Comparable<KeyWord>, QueryElement
{
    protected String keyword;

    public KeyWord()
    {
        this.keyword = getClass().getSimpleName();
    }

    public int compareTo( KeyWord o )
    {
        // longer first for parsing queries
        return o.keyword.length() - keyword.length();
    }

    public static Class checkFieldExists( Class entity, String property )
    {
        Class clazz = entity;
        Field attribute = ReflectionUtils.findField( entity, property );
        if ( attribute == null )
        {
            throw new IllegalStateException( "Entity " + entity.getSimpleName() + " has no attribute " + property );
        }
        return attribute.getType();
    }

    public void process( JPQLPersistenceQuery jpqlQuery, List<Class> params )
    {
        jpqlQuery.append( " " ).append( keyword.toLowerCase() );
    }

    public String getKeyword()
    {
        return keyword;
    }

    public String toString()
    {
        return keyword;
    }

    public boolean postProcess( ListIterator<QueryElement> elements )
    {
        return true;
    }
}